/*
 * The UDS Collection
 * Copyright (C) 2001, 2002  Julian v. Bock
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	(See file COPYING for more information)
 */

#ifndef _uds_streambuf_hh
#define _uds_streambuf_hh


// includes

#include <iostream>


namespace uds
{

using namespace std;


/** stream buffer class for file descriptors
	\author Julian v. Bock */
class fdbuf : public streambuf
{
public:
	/// default buffer size
	static const size_t buf_size = 0x1000;

	/** default constructor */
	fdbuf();

	/** destructor */
	virtual
	~fdbuf();

	/** buffer underflow */
	virtual int_type
	underflow();

	/** buffer overflow */
	virtual int_type
	overflow( int_type c = traits_type::eof() );

	/** sync buffers */
	virtual int_type
	sync() { return overflow( traits_type::eof() ); }

	/** attaches a file descriptor to the buffer
		\param _fd file descriptor
		\param mode mode */
	virtual void
	Attach( int _fd, ios::openmode mode = ios::in | ios::out );

	/** detaches the file descriptor currently used */
	void
	Detach();

	/** \return file descriptor */
	int
	File() const { return fd; }

protected:
	/** performs read operation
		\param buf buffer start
		\param len max. length
		\return nuber of bytes read */
	virtual int_type
	Read( char_type* buf, int_type len );

	/** performs write operation
		\param buf buffer start
		\param len length */
	virtual int_type
	Write( char_type* buf, int_type len );

	/** local input buffer access
		\return input buffer */
	char_type*
	libuf() { return ibuf; }

	/** local output buffer access
		\return output buffer */
	char_type*
	lobuf() { return obuf; }

private:
	/// file descriptor
	int			fd;
	/// input buffer
	char_type	ibuf[buf_size];
	/// output buffer
	char_type	obuf[buf_size];
};

}

#endif // _uds_streambuf_hh
