/*
 * The UDS Collection
 * Copyright (C) 2001, 2002  Julian v. Bock
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	(See file COPYING for more information)
 */

#ifndef _uds_stream_hh
#define _uds_stream_hh


// includes

#include <iostream>


namespace uds
{

using namespace std;


/** base class for input streams that provide a file descriptor
	\author Julian v. Bock */
class ifdstream : public istream
{
public:
	/** \param sb stream buffer */
	ifdstream( __streambuf_type* sb ) :
		istream( sb ) {}
	
	/** destructor */
	virtual
	~ifdstream();
	
	/** \return file descriptor */
	virtual int
	File() const = 0;
};


/** base class for output streams that provide a file descriptor
	\author Julian v. Bock */
class ofdstream : public ostream
{
public:
	/** \param sb stream buffer */
	ofdstream( __streambuf_type* sb ) :
		ostream( sb ) {}
	
	/** destructor */
	virtual
	~ofdstream();
	
	/** \return file descriptor */
	virtual int
	File() const = 0;
};


/** base class for input / output streams that provide a file descriptor
	\author Julian v. Bock */
class iofdstream : public ifdstream, public ofdstream
{
public:
	typedef ifdstream::__streambuf_type __streambuf_type;

	/** \param sb stream buffer */
	iofdstream( __streambuf_type* sb ) :
		ifdstream( sb ), ofdstream( sb ) {}
	
	/** destructor */
	virtual
	~iofdstream();
};
	
}

#endif // _uds_stream_hh
