/*
 * The UDS Collection
 * Copyright (C) 2000, 2001, 2002  Julian v. Bock
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	(See file COPYING for more information)
 */

#ifndef _uds_str_util_hh
#define _uds_str_util_hh


// includes

#include <uds/stdexcpt.hh>
#include <string>
#include <sstream>


namespace uds
{

using namespace std;


/** if the string s is longer than n characters, truncate the string and
	append e
	\param s string to truncate
	\param n max. length
	\param e string to append if s was truncated
	\return truncated string */
string
TrString( const string& s, size_t n, const string& e = "..." );

/** breaks up a string into several lines if it exceeds a certain width.
	The string should not contain a newline.
	\param l line
	\param w number of whitespaces to append after each break
	\param m max. line length
	\param f max. length of first line. if this parameter is set to 0, m - w is
	assumed
	\return new string */
string
BreakLine( const string& l, size_t w = 0, size_t m = 79, size_t f = 0 );

/** converts a variable to a string
	\param v value
	\return string representation of v */
template < class Tp >
string
StringVar( const Tp& v )
{
	stringstream s;
	s<<v;
	return s.str();
}

/** converts one type to another. if the 'strict' flag is set and not the
	whole string could be converted, an exception is thrown.
	\param v value
	\param strict 'strict' flag
	\return converted value
	\arg throws runtime_error */
template < class Ret, class Tp >
Ret
VarConv( const Tp& v, bool strict = true )
{
	stringstream s;
	s<<v<<' ';

	Ret ret;
	s>>ret;

	if ( strict && static_cast< int >( s.tellg() ) + 1 != s.tellp() )
	{
		string str = s.str();
		str = str.substr( 0, str.size() - 1 );
		
		Throw( runtime_error( "unable to convert whole string ["
							  + TrString( str, 20 ) + "]" ) );
	}
	
	return ret;
}

/** assigns one type to another. if the 'strict' flag is set and not the
	whole string could be converted, an exception is thrown.
	\param dst destination
	\param src source value
	\param strict 'strict' flag
	\arg throws runtime_error */
template < class Src, class Dst >
inline void
VarAssign( Dst& dst, const Src& src, bool strict = true )
{
	dst = VarConv< Dst >( src, strict );
}

/** quotes a string to protect it from shell expansion
	\param str string to quote
	\return quoted string */
string
ShellQuote( const string& str );

}

#endif // _uds_str_util_hh
