/*
 * The UDS Collection
 * Copyright (C) 2000, 2001, 2002  Julian v. Bock
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	(See file COPYING for more information)
 */

#ifndef _uds_uds_hh
#define _uds_uds_hh


namespace uds
{

using namespace std;


typedef unsigned char uds_flags_t;
extern uds_flags_t flags;

enum
{
	/// search for memory leaks. this flag can not be changed at runtime.
	leak_check			= 0x1,
	/// force core dumps when an uds exception is thrown
	force_core_dumps	= 0x2,
	/// create zombie objects
	summon_zombies		= 0x4,
	/// log memory (de)allocations
	log_allocs			= 0x8,
	/// rename uds core dumps
	rename_cores		= 0x10,
	/// generate always a backtrace when an uds exception is thrown
	std_backtraces		= 0x20,
};


/** UDS version class
	\author Julian v. Bock */
class Version_t
{
public:
	/** \param v version string */
	Version_t( const char* v );

	/** \param vmaj major version number
		\param vmin minor version number
		\param vpat patch level */
	Version_t( unsigned vmaj, unsigned vmin, unsigned vpat );

	/** \return major version number */
	unsigned
	Major() const { return ( ver & 0xff000000 ) >> 24; }

	/** \return minor version number */
	unsigned
	Minor() const { return ( ver & 0x00ff0000 ) >> 16; }

	/** \return patch level */
	unsigned
	Patch() const { return ver & 0x0000ffff; }

	/** \return encoded version number */
	unsigned
	Version() const { return ver; }

	bool
	operator < ( const Version_t& rhs ) { return ver < rhs.ver; }

private:
	/// version
	unsigned ver;

	/** \param vmaj major version number
		\param vmin minor version number
		\param vpat patch level */
	void
	init( unsigned vmaj, unsigned vmin, unsigned vpat );
};

/** \return the current version */
Version_t
Version();

}

#endif // _uds_sentinel_hh
