dnl checks for libuds

AC_DEFUN(UDS_CHECK_LIBUDS, [
AC_REQUIRE([UDS_URL])

uds_oldlibs=$LIBS
AC_CHECK_LIB(uds, main, , [
	echo
	echo "*** the UDS Collection was not found on your system"
	echo "*** see $uds_url_rs"
	echo
	exit 1
])
LIBS=$uds_oldlibs
])


dnl checks for libudsthread

AC_DEFUN(UDS_CHECK_LIBUDSTHREAD, [
AC_REQUIRE([UDS_CHECK_LIBUDS])

uds_oldlibs=$LIBS
AC_CHECK_LIB(udsthread, main, , [
	echo
	echo "*** libudsthread was not built on your system"
	echo "*** this library is essentially for the operation of this package"
	echo
	exit 1
])
LIBS=$uds_oldlibs
])


dnl checks for specific versions of uds
dnl
dnl arg1: min. version
dnl arg2: max. version (optional)

AC_DEFUN(UDS_CHECK_VERSION, [
AC_REQUIRE([UDS_CHECK_LIBUDS])

if test "$2"; then
	AC_MSG_CHECKING(for UDS version >= $1 <= $2)
	uds_mv="$2"
else
	AC_MSG_CHECKING(for UDS version >= $1)
	uds_mv="100.0.0"
fi

uds_oldlibs=$LIBS
LIBS=-luds
AC_TRY_RUN([
#include <uds/uds.hh>
int
main()
{
	return ( ( uds::Version() < uds::Version_t( "$1" ) )
		|| !( uds::Version() < uds::Version_t( "$uds_mv" ) ))
		? 1 : 0;
}
],
	AC_MSG_RESULT(found)
, [
	AC_MSG_WARN(not found)
	echo
	echo "*** you need a newer version of libuds"
	echo "*** see $uds_url_rs"
	echo
	exit 1
], [
	AC_MSG_RESULT(cross compiling: test not performed)
] )
LIBS=$uds_oldlibs
])


dnl checks for a sufficient version of the C++ compiler

AC_DEFUN(UDS_CHECK_CXX, [
AC_REQUIRE([AC_PROG_CXX])

AC_MSG_CHECKING(for C++ compiler version)
uds_gcc_ver=$($CXX --version 2>/dev/null | head -n 1)
AC_MSG_RESULT($uds_gcc_ver)
AC_DEFINE_UNQUOTED(UDS_CXX_VERSION,"$uds_gcc_ver",C++ compiler version)

changequote(, )
test $(echo $uds_gcc_ver | sed 's/^[^0-9]*\(.\).*/\1/') -lt "3" && {
	echo
	echo "*** If you are using the GNU Compiler Collection,"
	echo "*** your version of gcc is too old."
	echo "*** Upgrade to gcc 3.0 or higher."
	echo
}
changequote([, ])

])
